# $Id: RottenTomatoes.pm 3740 2010-01-19 12:31:58Z chris $

=head1 NAME

WebService::RottenTomatoes - OO Perl interface to the movie rating site rottentomatoes.com

=head1 SYNOPSIS

    TODO

=head1 DESCRIPTION

    TODO

=cut

package WebService::RottenTomatoes;

use strict;
use warnings;

our $VERSION = '0.01';

use Cache::FileCache;

use Carp;

use File::Spec::Functions qw(tmpdir);

use LWP::UserAgent;

use WebService::RottenTomatoes::Movie;
use WebService::RottenTomatoes::Celebrity;
use WebService::RottenTomatoes::Author;
use WebService::RottenTomatoes::Source;

=head1 METHODS

=head2 new(%opts)

Constructor.

%opts can contain:

=over 4

=item cache - Whether to cache responses.  Defaults to true

=item cache_root - The root dir for the cache.  Defaults to tmpdir();

=item cache_exp - How long to cache responses for.  Defaults to "1h"

=back

=cut

sub new {
    my $class = shift;
    my %args = @_;
    my $self = {};
    
    bless $self, $class;
    
    $self->{'_cache_root'} = $args{'cache_root'} || tmpdir();
    $self->{'_cache_exp'} = $args{'cache_exp'} || "1h";
    $self->{'_cache'} = defined $args{'cache'} ? $args{'cache'} : 1;

    if ($self->{'_cache'}) {
	$self->{'_cacheObj'} = Cache::FileCache->new( {'cache_root' => $self->{'_cache_root'}, 'namespace' => "WebService-RottenTomatoes", 'default_expires_in' => $self->{'_cache_exp'}} );
    }

    $self->{'_useragent'} = LWP::UserAgent->new();
    $self->{'_useragent'}->env_proxy();
    $self->{'_useragent'}->agent("WebService::RottenTomatoes/$VERSION");

    return $self;
}


=head2 search(%args)

TODO

%args can contain:

=over 4

=item type - Resource type: "Movie", "Celebrity", "Author", "Source"

=item rtname - RottenTomatoes string id (Movie*)

=item rtid - RottenTomatoes id number (Movie, Celebrity*, Author*, Source*)

=item imdbid - IMDB identifier e.g. "tt0000001", "nm0000001" (Movie)

=item imdbtitle - IMDB official title e.g. "Almost Famous (2000)"

=back

=cut

sub search {
    my $self = shift;
    my %args = @_;

    if ($args{'type'} eq "Movie") {
	return WebService::RottenTomatoes::Movie->_new($self, %args);
    } elsif ($args{'type'} eq "Celebrity") {
	return WebService::RottenTomatoes::Celebrity->_new($self, %args);
    } elsif ($args{'type'} eq "Author") {
	return WebService::RottenTomatoes::Author->_new($self, %args);
    } elsif ($args{'type'} eq "Source") {
	return WebService::RottenTomatoes::Source->_new($self, %args);
    } else {
	croak "Unknown resource type '$args{'type'}'";
    }	

}

sub _get_page {
    my $self = shift;
    my $url = shift;

    my $cacheKey = "URL-RESPONSE:" . $url;

    my $response;

    if ($self->{'_cache'}) {
	$response = $self->{'_cacheObj'}->get($cacheKey);
    }

    if (! defined $response) {

	$response = $self->{'_useragent'}->get($url);
    
	if($response->code() ne "200") {
	    croak "URL (", $url, ") Request Failed - Code: ", $response->code(), " Error: ", $response->message(), "\n";
	}

	if ($self->{'_cache'}) {
	    $self->{'_cacheObj'}->set($cacheKey, $response);
	}
    }

    return $response;

}

sub _get_page_content {
    my $self = shift;
    my $url = shift;

    my $cacheKey = "URL-DECODED-CONTENT:" . $url;

    my $content;

    if ($self->{'_cache'}) {
	$content = $self->{'_cacheObj'}->get($cacheKey);
    }

    if (! defined $content) {
	my $response = $self->_get_page($url);

	$content = $response->decoded_content();

	if ($self->{'_cache'}) {
	    $self->{'_cacheObj'}->set($cacheKey, $content);
	}
    }

    return $content;
}

sub domain {
    return "uk.rottentomatoes.com";
}

1;
